package com.example.bdkipoc;

import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class TransactionActivity extends AppCompatActivity implements TransactionAdapter.OnPrintClickListener {
    private RecyclerView recyclerView;
    private TransactionAdapter adapter;
    private List<Transaction> transactionList;
    private List<Transaction> filteredList;
    private ProgressBar progressBar;
    private EditText searchEditText;
    private ImageButton searchButton;
    
    // Pagination variables
    private int page = 0;
    private final int limit = 10;
    private boolean isLoading = false;
    private boolean isLastPage = false;
    private String currentSearchQuery = "";
    private boolean isRefreshing = false;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_transaction);

        // Initialize views
        initViews();
        
        // Setup toolbar
        setupToolbar();
        
        // Setup RecyclerView
        setupRecyclerView();
        
        // Setup search functionality
        setupSearch();
        
        // Load initial data
        loadTransactions(0);
    }

    private void initViews() {
        recyclerView = findViewById(R.id.recyclerView);
        progressBar = findViewById(R.id.progressBar);
        searchEditText = findViewById(R.id.searchEditText);
        searchButton = findViewById(R.id.searchButton);
    }

    private void setupToolbar() {
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        if (getSupportActionBar() != null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
            getSupportActionBar().setDisplayShowHomeEnabled(true);
        }
    }

    private void setupRecyclerView() {
        transactionList = new ArrayList<>();
        filteredList = new ArrayList<>();
        
        adapter = new TransactionAdapter(filteredList);
        adapter.setPrintClickListener(this);
        
        LinearLayoutManager layoutManager = new LinearLayoutManager(this);
        recyclerView.setLayoutManager(layoutManager);
        recyclerView.setAdapter(adapter);

        // Add scroll listener for pagination
        recyclerView.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                
                // Pagination: Load more when reaching the bottom
                if (!recyclerView.canScrollVertically(1) && !isLoading && !isLastPage && currentSearchQuery.isEmpty()) {
                    loadTransactions(page + 1);
                }
                
                // Detect scroll to top for refresh gesture (double tap on top)
                LinearLayoutManager layoutManager = (LinearLayoutManager) recyclerView.getLayoutManager();
                if (layoutManager != null && layoutManager.findFirstCompletelyVisibleItemPosition() == 0 && dy < 0) {
                    // User is at top and scrolling up, enable refresh on search button long press
                }
            }
        });
    }

    private void setupSearch() {
        // Search button click listener
        searchButton.setOnClickListener(v -> performSearch());
        
        // Search button long press for refresh
        searchButton.setOnLongClickListener(v -> {
            refreshTransactions();
            Toast.makeText(this, "Refreshing data...", Toast.LENGTH_SHORT).show();
            return true;
        });
        
        // Search EditText listener
        searchEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {}

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                currentSearchQuery = s.toString().trim();
                filterTransactions(currentSearchQuery);
            }

            @Override
            public void afterTextChanged(Editable s) {}
        });
    }

    private void refreshTransactions() {
        if (isRefreshing) return;
        
        isRefreshing = true;
        // Clear search when refreshing
        searchEditText.setText("");
        currentSearchQuery = "";
        page = 0;
        isLastPage = false;
        transactionList.clear();
        filteredList.clear();
        adapter.notifyDataSetChanged();
        loadTransactions(0);
    }

    private void performSearch() {
        String query = searchEditText.getText().toString().trim();
        currentSearchQuery = query;
        filterTransactions(query);
        
        // Hide keyboard
        searchEditText.clearFocus();
    }

    private void filterTransactions(String query) {
        filteredList.clear();
        
        if (query.isEmpty()) {
            filteredList.addAll(transactionList);
        } else {
            for (Transaction transaction : transactionList) {
                if (transaction.referenceId.toLowerCase().contains(query.toLowerCase()) ||
                    transaction.amount.contains(query)) {
                    filteredList.add(transaction);
                }
            }
        }
        
        adapter.notifyDataSetChanged();
        
        // Scroll to top after filtering
        if (!filteredList.isEmpty()) {
            recyclerView.scrollToPosition(0);
        }
    }

    private void loadTransactions(int pageToLoad) {
        isLoading = true;
        if (pageToLoad == 0) {
            progressBar.setVisibility(View.VISIBLE);
        }
        new FetchTransactionsTask(pageToLoad).execute();
    }

    private class FetchTransactionsTask extends AsyncTask<Void, Void, List<Transaction>> {
        private int pageToLoad;
        private boolean error = false;
        private int total = 0;

        FetchTransactionsTask(int page) {
            this.pageToLoad = page;
        }

        @Override
        protected List<Transaction> doInBackground(Void... voids) {
            List<Transaction> result = new ArrayList<>();
            try {
                String urlString = "https://be-edc.msvc.app/transactions?page=" + pageToLoad + "&limit=" + limit + "&sortOrder=DESC&from_date=&to_date=&location_id=0&merchant_id=0&tid=73001500&mid=71000026521&sortColumn=id";
                URI uri = new URI(urlString);
                URL url = uri.toURL();
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET");
                conn.setRequestProperty("accept", "*/*");
                int responseCode = conn.getResponseCode();
                if (responseCode == 200) {
                    BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
                    StringBuilder response = new StringBuilder();
                    String line;
                    while ((line = in.readLine()) != null) {
                        response.append(line);
                    }
                    in.close();
                    JSONObject jsonObject = new JSONObject(response.toString());
                    JSONObject results = jsonObject.getJSONObject("results");
                    total = results.getInt("total");
                    JSONArray data = results.getJSONArray("data");
                    for (int i = 0; i < data.length(); i++) {
                        JSONObject t = data.getJSONObject(i);
                        Transaction tx = new Transaction(
                                t.getInt("id"),
                                t.getString("type"),
                                t.getString("channel_category"),
                                t.getString("channel_code"),
                                t.getString("reference_id"),
                                t.getString("amount"),
                                t.getString("cashflow"),
                                t.getString("status"),
                                t.getString("created_at"),
                                t.getString("merchant_name")
                        );
                        result.add(tx);
                    }
                } else {
                    error = true;
                }
            } catch (IOException | JSONException | URISyntaxException e) {
                error = true;
            }
            return result;
        }

        @Override
        protected void onPostExecute(List<Transaction> transactions) {
            isLoading = false;
            isRefreshing = false;
            progressBar.setVisibility(View.GONE);
            
            if (error) {
                Toast.makeText(TransactionActivity.this, "Failed to fetch transactions", Toast.LENGTH_SHORT).show();
                return;
            }
            
            if (pageToLoad == 0) {
                transactionList.clear();
            }
            transactionList.addAll(transactions);
            
            // Update filtered list based on current search
            filterTransactions(currentSearchQuery);
            
            page = pageToLoad;
            if (transactionList.size() >= total) {
                isLastPage = true;
            }
            
            // Scroll to top if it's a refresh
            if (pageToLoad == 0 && !filteredList.isEmpty()) {
                recyclerView.scrollToPosition(0);
            }
        }
    }

    @Override
    public void onPrintClick(Transaction transaction) {
        // Handle print button click
        Toast.makeText(this, "Cetak Ulang: " + transaction.referenceId, Toast.LENGTH_SHORT).show();
        
        // Implement your print logic here
        // For example, you might want to:
        // 1. Generate receipt
        // 2. Send to printer
        // 3. Show print preview
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    // Transaction model class
    static class Transaction {
        int id;
        String type;
        String channelCategory;
        String channelCode;
        String referenceId;
        String amount;
        String cashflow;
        String status;
        String createdAt;
        String merchantName;

        Transaction(int id, String type, String channelCategory, String channelCode, String referenceId, String amount, String cashflow, String status, String createdAt, String merchantName) {
            this.id = id;
            this.type = type;
            this.channelCategory = channelCategory;
            this.channelCode = channelCode;
            this.referenceId = referenceId;
            this.amount = amount;
            this.cashflow = cashflow;
            this.status = status;
            this.createdAt = createdAt;
            this.merchantName = merchantName;
        }
    }
}